﻿using System.Windows;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Benchmark.Utils;

namespace Dapfor.Wpf.Grid.Benchmark
{
    /// <summary>
    /// Interaction logic for AddRowControl.xaml
    /// </summary>
    public partial class AddRowControl : IActivable
    {
        private static readonly HighPerformanceTimer HiPerfTimer = new HighPerformanceTimer();
        private bool _stopping;

        public AddRowControl()
        {
            InitializeComponent();
        }

        #region IActivable Members

        public void Activate()
        {
            controlPanel.IsEnabled = true;
        }

        public void Deactivate()
        {
            _stopping = true;
            controlPanel.IsEnabled = false;
        }

        #endregion


        private void OnStartButtonClick(object sender, RoutedEventArgs e)
        {
            controlPanel.IsEnabled = false;
            _stopping = false;
            Helper.DoEvents();

            //Remove sort
            grid.Headers[0].SortedColumns.Clear();

            //Remove already added rows
            if (grid.Rows.Count > 0)
            {
                Logger.Log("Prepare Wpf Grid and remove all existing rows");
                grid.Rows.Clear();
            }

            int rowCount = 500000;
            int tranche = 10000;
            //Limit amount of adding data
            if (rbToEnd.IsChecked == true)
            {
                Logger.Log(string.Format("Performance measurement when rows are inserted to the end of grid"));
                rowCount = 500000;
                tranche = 10000;
            }
            else if (rbToBeginning.IsChecked == true)
            {
                Logger.Log(string.Format("Performance measurement when rows are inserted to the beginning of grid"));
                rowCount = 2000;
                tranche = 50;
            }
            else if (rbToSortedGrid.IsChecked == true)
            {
                Logger.Log(string.Format("Performance measurement when rows are inserted to sorted grid"));
                rowCount = 100000;
                tranche = 10000;
                grid.Headers[0][1].SortDirection = SortDirection.Ascending;
                grid.Headers[0][3].SortDirection = SortDirection.Descending;
            }

          
            HiPerfTimer.Start();

            for (int i = 0; (i < rowCount) && !_stopping; ++i)
            {

                //Adding to the end or to the sorted container
                Row row = grid.Rows.Add(new TestDataObject());

                //Adding to the beginning
                if (rbToBeginning.IsChecked == true)
                {
                    row.ChildIndex = 0;
                }

                if ((i + 1) % tranche == 0 && i > 0)
                {
                    HiPerfTimer.Stop();


                    double duration = HiPerfTimer.Duration;

                    int rate = duration > 0 ? (int)(1000 * tranche / duration) : 0;

                    Logger.Log(string.Format("Rows {0} - {1} added in {2:0} msec. Rate: {3} rows/sec", i + 1 - tranche, i, duration, rate));
                    Helper.DoEvents();

                    HiPerfTimer.Start();
                }
            }
            

            HiPerfTimer.Stop();

            controlPanel.IsEnabled = true;
            Logger.Log(string.Format("Adding completed"));
        }
    }
}