﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Benchmark.Utils;
using Brushes = Dapfor.Wpf.Grid.Benchmark.Utils.Brushes;

namespace Dapfor.Wpf.Grid.Benchmark
{
    /// <summary>
    /// Interaction logic for DataUpdatingNonEventDrivenModelControl.xaml
    /// </summary>
    public partial class DataUpdatingNonEventDrivenModelControl : IActivable
    {
        private readonly Random _random = new Random();
        private bool _measuring;
        private DateTime _measureStart;
        private int _updatesCount;
        private readonly List<int> _updateScenarios = new List<int>();
        private readonly PerformanceCounter _cpuCounter = new PerformanceCounter();
        private DateTime _lastStat = DateTime.UtcNow;
        private int _lastStatCount;


        public DataUpdatingNonEventDrivenModelControl()
        {
            InitializeComponent();

            //Initialize performance counters
            _cpuCounter.CategoryName = "Processor";
            _cpuCounter.CounterName = "% Processor Time";
            _cpuCounter.InstanceName = "_Total";

            grid.ProcessUpdate += OnUpdateCell;

            
            grid.SettingsBlinking.Enabled = false;
            /*grid.RowUpdated += delegate(object sender, GridControlRowUpdateEventArgs e)
            {
                Brush color = r.Next()%2 == 0 ? Brushes.Green : Brushes.Red;
                e.Row[e.DataField.Id].Highlight(TimeSpan.FromMilliseconds(1000), Brushes.Red);

            };*/
        }

        private void OnUpdateCell(object sender, EventArgs e)
        {
            if (grid.IsLimitFound && !_measuring)
            {
                _updatesCount = 0;
                _measuring = true;
                _measureStart = DateTime.UtcNow;
                _lastStatCount = 0;
                _lastStat = DateTime.UtcNow;
            }


            //Updates random cell in the grid;
            Row row = grid.Rows.Count > 0 ? grid.Rows[_random.Next() % grid.Rows.Count] : null;
            Cell cell = row != null ? row[_random.Next() % grid.Headers[0].Count] : null;
            Brush color = _random.Next() % 2 == 0 ? Brushes.Green : Brushes.Red;

            if (cell != null)
            {
                int value = _random.Next();
                if (cbNonGuiThread.IsChecked == true)
                {
                    ThreadPool.QueueUserWorkItem(delegate
                    {
                        cell.Value = value;
                        cell.Blink(TimeSpan.FromMilliseconds(1000), color);
                    });
                }
                else
                {
                    cell.Value = value;
                    cell.Blink(TimeSpan.FromMilliseconds(1000), color);
                }
            }

            _updatesCount++;
            if (_measuring)
            {
                //Measure refresh rate for 5 seconds
                if (DateTime.UtcNow - _measureStart > TimeSpan.FromSeconds(5))
                {
                    grid.StopLoadBalancingProcess();

                    int visibleRows = grid.Rows.BottomVisibleRow != null && grid.Rows.TopVisibleRow != null
                        ? (grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex)
                        : 0;
                    long visibleCells = grid.Headers[0].Count * visibleRows;
                    long totalCells = grid.Headers[0].Count * grid.Rows.Count;

                    double rate = totalCells != 0 ? (double)visibleCells / totalCells : 0;


                    Logger.Log(string.Format("Grid of {0} rows, size = {2:0}x{3:0}. Maximum refresh rate: {1} updates/sec, in visible bounds: {4:0} updates/sec",
                                              grid.Rows.Count,
                                              _updatesCount / 5,
                                              grid.ActualWidth,
                                              grid.ActualHeight,
                                              rate * _updatesCount / 5));

                    //Execute the next scenario
                    StartTest();
                }
            }

            DisplayStatistics();

        }

        private void DisplayStatistics()
        {
            if (DateTime.UtcNow - _lastStat > TimeSpan.FromMilliseconds(500))
            {
                lbCPULoadRate.Text = string.Format("CPU load rate: {0:0} %", _cpuCounter.NextValue());
                lbGridSize.Text = string.Format("Grid size: {0:0}x{1:0}", grid.ActualWidth, grid.ActualHeight);
                double updates = 1000 * (_updatesCount - _lastStatCount) / (DateTime.UtcNow - _lastStat).TotalMilliseconds;
                lbUpdateRate.Text = string.Format("Update rate: {0:0} updates/sec", updates);

                int visibleRows = grid.Rows.BottomVisibleRow != null && grid.Rows.TopVisibleRow != null
                        ? (grid.Rows.BottomVisibleRow.VisibleIndex - grid.Rows.TopVisibleRow.VisibleIndex)
                        : 0;
                long visibleCells = grid.Headers[0].Count * visibleRows;
                long totalCells = grid.Headers[0].Count * grid.Rows.Count;

                double rate = totalCells != 0 ? (double)visibleCells / totalCells : 0;
                lbVisibleBoundsUpdateRate.Text = string.Format("Update rate in visible bounds: {0:0} updates/sec", rate * updates);

                gbStatistics.Visibility = Visibility.Visible;

                _lastStatCount = _updatesCount;
                _lastStat = DateTime.UtcNow;
            }
            
        }

        public void Activate()
        {
            panel.IsEnabled = true;
        }

        public void Deactivate()
        {
            panel.IsEnabled = false;
            grid.StopLoadBalancingProcess();
            _updatesCount = 0;
            _measuring = true;
            _measureStart = DateTime.UtcNow;
            _lastStatCount = 0;
            _lastStat = DateTime.UtcNow;
            gbStatistics.Visibility = Visibility.Hidden;
            grid.Rows.Clear();
        }

        private void OnStartButtonClick(object sender, RoutedEventArgs e)
        {
            _updateScenarios.Add(10);
            _updateScenarios.Add(20);
            _updateScenarios.Add(50);
            _updateScenarios.Add(100);
            _updateScenarios.Add(200);
            _updateScenarios.Add(500);
            _updateScenarios.Add(1000);
            _updateScenarios.Add(5000);
            _updateScenarios.Add(10000);
            _updateScenarios.Add(20000);
            _updateScenarios.Add(50000);
            _updateScenarios.Add(100000);

            Logger.Log(string.Format("Application measures performance of Wpf Grid. It may take a little time..."));

            StartTest();
        }

        private void StartTest()
        {
            if (_updateScenarios.Count > 0)
            {
                panel.IsEnabled = false;
                int rowsCount = _updateScenarios[0];
                _updateScenarios.RemoveAt(0);

                grid.StopLoadBalancingProcess();

                grid.Headers[0].SortedColumns.Clear();


                //Populate the grid
                grid.Rows.Clear();
                for (int i = 0; i < rowsCount; ++i)
                {
                    grid.Rows.Add(new[] {_random.Next(), _random.Next(), _random.Next(), _random.Next(), _random.Next(), _random.Next(), _random.Next(), _random.Next(), _random.Next(), _random.Next()});
                }

                if (cbEnableSort.IsChecked == true)
                {
                    grid.Headers[0][1].SortDirection = SortDirection.Ascending;
                    grid.Headers[0][3].SortDirection = SortDirection.Descending;
                }

                _measuring = false;

                grid.StartLoadBalancingProcess();
            }
            else
            {
                _measuring = false;
                panel.IsEnabled = true;

                grid.StopLoadBalancingProcess();
                panel.IsEnabled = true;

                Logger.Log(string.Format("Test completed"));
            }


        }
    }
}